@@ -1,12 +1,13 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/libncdap3/dapdump.c,v 1.17 2009/11/17 17:22:47 dmh Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/libncdap3/dapdump.c,v 1.21 2010/05/26 21:43:31 dmh Exp $
  *********************************************************************/
 #include "config.h"
-#include "netcdf3.h"
+#ifdef USE_PARALLEL
+#include "netcdf_par.h"
+#endif
 #include "ncdap3.h"
-#include "daprename.h"
 #include "dapdump.h"
 
 #define CHECK(n) if((n) != NC_NOERR) {return (n);} else {}
@@ -22,7 +23,7 @@ dumpmetadata(int ncid, NChdr** hdrp)
     hdr->content = ncbytesnew();
     if(hdrp) *hdrp = hdr;
 
-    stat = RENAME3(inq)(hdr->ncid,
+    stat = nc_inq(hdr->ncid,
 		  &hdr->ndims,
 		  &hdr->nvars,
 		  &hdr->ngatts,
@@ -43,12 +44,13 @@ dumpmetadata(int ncid, NChdr** hdrp)
 	size_t typesize;
         size_t nvalues;
 
-        stat = RENAME3(inq_attname)(hdr->ncid,NC_GLOBAL,i,attname);
+        stat = nc_inq_attname(hdr->ncid,NC_GLOBAL,i,attname);
         CHECK(stat);
 	att->name = nulldup(attname);
-	stat = RENAME3(inq_att)(hdr->ncid,NC_GLOBAL,att->name,&nctype,&nvalues);
+	stat = nc_inq_att(hdr->ncid,NC_GLOBAL,att->name,&nctype,&nvalues);
         CHECK(stat);
 	att->etype = nctypetodap(nctype);
+ 	typesize = nctypesizeof(att->etype);
 	fprintf(stdout,"\t[%d]: name=%s type=%s values(%lu)=",
 			i,att->name,nctypetostring(octypetonc(att->etype)),
                         (unsigned long)nvalues);
@@ -56,7 +58,7 @@ dumpmetadata(int ncid, NChdr** hdrp)
 	    size_t len = typesize*nvalues;
 	    char* values = (char*)emalloc(len+1);/* for null terminate*/
 	    MEMCHECK(values,NC_ENOMEM);
-	    stat = RENAME3(get_att)(hdr->ncid,NC_GLOBAL,att->name,values);
+	    stat = nc_get_att(hdr->ncid,NC_GLOBAL,att->name,values);
             CHECK(stat);
 	    values[len] = '\0';
 	    fprintf(stdout," '%s'",values);
@@ -64,7 +66,7 @@ dumpmetadata(int ncid, NChdr** hdrp)
 	    size_t len = typesize*nvalues;
 	    char* values = (char*)emalloc(len);
 	    MEMCHECK(values,NC_ENOMEM);
-	    stat = RENAME3(get_att)(hdr->ncid,NC_GLOBAL,att->name,values);
+	    stat = nc_get_att(hdr->ncid,NC_GLOBAL,att->name,values);
             CHECK(stat);
 	    for(k=0;k<nvalues;k++) {
 		fprintf(stdout," ");
@@ -78,7 +80,7 @@ dumpmetadata(int ncid, NChdr** hdrp)
     MEMCHECK(hdr->dims,NC_ENOMEM);
     for(i=0;i<hdr->ndims;i++) {
 	hdr->dims[i].dimid = i;
-        stat = RENAME3(inq_dim)(hdr->ncid,
+        stat = nc_inq_dim(hdr->ncid,
 	                  hdr->dims[i].dimid,
 	                  hdr->dims[i].name,
 	                  &hdr->dims[i].size);
@@ -92,7 +94,7 @@ dumpmetadata(int ncid, NChdr** hdrp)
 	Var* var = &hdr->vars[i];
 	nc_type nctype;
 	var->varid = i;
-        stat = RENAME3(inq_var)(hdr->ncid,
+        stat = nc_inq_var(hdr->ncid,
 	                  var->varid,
 	                  var->name,
 			  &nctype,
@@ -120,16 +122,16 @@ dumpmetadata(int ncid, NChdr** hdrp)
 	    char* values;
 	    nc_type nctype;
 	    size_t nvalues;
-            stat = RENAME3(inq_attname)(hdr->ncid,var->varid,j,attname);
+            stat = nc_inq_attname(hdr->ncid,var->varid,j,attname);
 	    CHECK(stat);
 	    att->name = nulldup(attname);
-	    stat = RENAME3(inq_att)(hdr->ncid,var->varid,att->name,&nctype,&nvalues);
+	    stat = nc_inq_att(hdr->ncid,var->varid,att->name,&nctype,&nvalues);
 	    CHECK(stat);
 	    att->etype = nctypetodap(nctype);
 	    typesize = nctypesizeof(att->etype);
 	    values = (char*)emalloc(typesize*nvalues);
 	    MEMCHECK(values,NC_ENOMEM);
-	    stat = RENAME3(get_att)(hdr->ncid,var->varid,att->name,values);
+	    stat = nc_get_att(hdr->ncid,var->varid,att->name,values);
             CHECK(stat);
 	    fprintf(stdout,"\tattr[%d]: name=%s type=%s values(%lu)=",
 			j,att->name,nctypetostring(octypetonc(att->etype)),(unsigned long)nvalues);
@@ -201,7 +203,7 @@ dumpprojections(NClist* projections)
     for(i=0;i<nclistlength(projections);i++) {
 	NCprojection* p = (NCprojection*)nclistget(projections,i);
         if(i > 0) ncbytescat(buf,",");
-	ncbytescat(buf,dumpprojection1(p));
+	ncbytescat(buf,dumpprojection(p));
     }
     pstring = ncbytesdup(buf);
     ncbytesfree(buf);
@@ -209,19 +211,20 @@ dumpprojections(NClist* projections)
 }
 
 char*
-dumpprojection1(NCprojection* p)
+dumpprojection(NCprojection* p)
 {
     int i;
     NCbytes* buf;
     char* pstring;
     if(p == NULL) return nulldup("");
+    if(p->discrim == NS_FCN) return nulldup("?fcn()");
     buf = ncbytesnew();
-    for(i=0;i<nclistlength(p->segments);i++) {
-        NCsegment* segment = (NCsegment*)nclistget(p->segments,i);
+    for(i=0;i<nclistlength(p->var->segments);i++) {
+        NCsegment* segment = (NCsegment*)nclistget(p->var->segments,i);
 	char tmp[1024];
         snprintf(tmp,sizeof(tmp),"%s%s/%lu",
 	         (i > 0?".":""),
-	         (segment->segment?segment->segment:"<unknown>"),
+	         (segment->name?segment->name:"<unknown>"),
 		 (unsigned long)segment->slicerank);
 	ncbytescat(buf,tmp);
 	if(segment->slicesdefined)
@@ -231,7 +234,7 @@ dumpprojection1(NCprojection* p)
     }
     if(iswholeprojection(p)) ncbytescat(buf,"*");
     ncbytescat(buf,"(");
-    if(p->leaf != NULL) ncbytescat(buf,p->leaf->name);
+    if(p->var->leaf != NULL) ncbytescat(buf,p->var->leaf->name);
     ncbytescat(buf,")");
     pstring = ncbytesdup(buf);
     ncbytesfree(buf);
@@ -251,7 +254,7 @@ dumpselections(NClist* selections)
     if(nclistlength(selections) == 0) return nulldup("");
     for(i=0;i<nclistlength(selections);i++) {
 	NCselection* sel = (NCselection*)nclistget(selections,i);
-	ncbytescat(buf,dumpselection1(sel));
+	ncbytescat(buf,dumpselection(sel));
     }
     sstring = ncbytesdup(buf);
     ncbytesfree(buf);
@@ -259,56 +262,56 @@ dumpselections(NClist* selections)
 }
 
 char*
-dumpselection1(NCselection* sel)
+dumpselection(NCselection* sel)
 {
     NCbytes* buf = ncbytesnew();
+    NCbytes* segbuf = ncbytesnew();
     char* sstring;
-    NClist* path = NULL;
-    char* pathstring = NULL;
+    NClist* segments = NULL;
     int j;
 
     if(sel == NULL) return nulldup("");
-    path = sel->path;
+    segments = sel->lhs->var->segments;
     ncbytescat(buf,"&");
-    if(path == NULL)
-	pathstring = makecdfpathstring3(sel->node,".");
-    else
-	pathstring = simplepathstring3(path,".");
-    ncbytescat(buf,pathstring);
-    efree(pathstring);
+    tostringncsegments(segments,segbuf);
+    ncbytescat(buf,ncbytescontents(segbuf));
     ncbytescat(buf,opstrings[sel->operator]);
     ncbytescat(buf,"{");
-    for(j=0;j<nclistlength(sel->values);j++) {
-        NCvalue* value = (NCvalue*)nclistget(sel->values,j);
+    for(j=0;j<nclistlength(sel->rhs);j++) {
+        NCvalue* value = (NCvalue*)nclistget(sel->rhs,j);
+        NCconstant* con = value->constant;
         char tmp[64];
         if(j > 0) ncbytescat(buf,",");
-        switch (value->kind) {
-        case ST_STR:
-            ncbytescat(buf,value->value.text);
+        switch (value->discrim) {
+        case NS_STR:
+            ncbytescat(buf,con->text);
             break;          
-        case ST_INT:
-            snprintf(tmp,sizeof(tmp),"%lld",value->value.intvalue);
-            ncbytescat(buf,tmp);
+	case NS_CONST:
+            switch (con->discrim) {	    
+            case NS_INT:
+                snprintf(tmp,sizeof(tmp),"%lld",con->intvalue);
+                ncbytescat(buf,tmp);
+                break;
+            case NS_FLOAT:
+                snprintf(tmp,sizeof(tmp),"%g",con->floatvalue);
+                ncbytescat(buf,tmp);
+                break;
+            default: PANIC1("unexpected discriminator %d",(int)con->discrim);
+	    }
+	    break;
+        case NS_VAR:
+            segments = value->var->segments;
+	    ncbytesclear(segbuf);
+	    tostringncsegments(segments,segbuf);
+            ncbytescat(buf,ncbytescontents(segbuf));
             break;
-        case ST_FLOAT:
-            snprintf(tmp,sizeof(tmp),"%g",value->value.floatvalue);
-            ncbytescat(buf,tmp);
-            break;
-        case ST_VAR:
-            path = value->value.var.path;
-            if(path == NULL)
-                pathstring = makecdfpathstring3(value->value.var.node,".");
-            else
-                pathstring = simplepathstring3(path,".");
-            ncbytescat(buf,pathstring);
-            efree(pathstring);
-            break;
-        default: PANIC1("unexpected tag: %d",(int)value->kind);
+        default: PANIC1("unexpected discriminator %d",(int)value->discrim);
         }
     }
     ncbytescat(buf,"}");
     sstring = ncbytesdup(buf);
     ncbytesfree(buf);
+    ncbytesfree(segbuf);
     return sstring;
 }
 
@@ -317,6 +320,7 @@ dumpconstraint(NCconstraint* con)
 {
     NCbytes* buf = ncbytesnew();
     char* result = NULL;
+    if(con == NULL) {ncbytescat(buf,"null"); goto done;}
     if(nclistlength(con->projections)==0 && nclistlength(con->selections)==0)
 	goto done;
     if(nclistlength(con->projections) > 0)  {
@@ -358,7 +362,7 @@ dumpsegment(NCsegment* segment)
     char* result;
     if(segment == NULL) return nulldup("(nullsegment)");
     buf = ncbytesnew();
-    ncbytescat(buf,(segment->segment?segment->segment:"(null)"));
+    ncbytescat(buf,(segment->name?segment->name:"(null)"));
     if(!segment->slicesdefined)
 	ncbytescat(buf,dumpslices(segment->slices,segment->slicerank));
     result = ncbytesdup(buf);
@@ -596,26 +600,6 @@ dumpnode(CDFnode* node)
         snprintf(tmp,sizeof(tmp),"    }\n");
         ncbytescat(buf,tmp);
     }
-#ifdef IGNORE
-    for(i=0;i<nclistlength(node->array.ncdimensions);i++) {
-	CDFnode* dim = (CDFnode*)nclistget(node->array.ncdimensions,i);
-        snprintf(tmp,sizeof(tmp),"ncdims[%d]={\n",i);
-        ncbytescat(buf,tmp);
-	snprintf(tmp,sizeof(tmp),"    name=%s\n",dim->name);
-	ncbytescat(buf,tmp);
-	snprintf(tmp,sizeof(tmp),"    dimflags=%u\n",
-			(unsigned int)dim->dim.dimflags);
-        ncbytescat(buf,tmp);
-	snprintf(tmp,sizeof(tmp),"    declsize=%lu\n",
-		    (unsigned long)dim->dim.declsize);
-        ncbytescat(buf,tmp);
-	snprintf(tmp,sizeof(tmp),"    declsize0=%lu\n",
-		    (unsigned long)dim->dim.declsize0);
-        ncbytescat(buf,tmp);
-        snprintf(tmp,sizeof(tmp),"    }\n");
-        ncbytescat(buf,tmp);
-    }
-#endif
 
     result = ncbytesdup(buf);
     ncbytesfree(buf);
@@ -651,7 +635,7 @@ dumpcachenode(NCcachenode* node)
 		node->prefetch?"*":"",
 		(unsigned long)node,
 		(unsigned long)node->xdrsize,
-		makeconstraintstring3(&node->constraint));
+		buildconstraintstring3(node->constraint));
     ncbytescat(buf,tmp);
     if(nclistlength(node->vars)==0)
 	ncbytescat(buf,"null");
@@ -699,7 +683,27 @@ dumpcache(NCcache* cache)
     return result;
 }
 
-/* This should be consistent with makeslicestrin3 in constraints3.c */
+/* This should be consistent with makeslicestring3 in constraints3.c */
+char*
+dumpslice(NCslice* slice)
+{
+    static char tmp[1024];
+    if(slice->count== 1) {
+        snprintf(tmp,sizeof(tmp),"[%lu]",
+	        (unsigned long)slice->first);
+    } else if(slice->stride == 1) {
+        snprintf(tmp,sizeof(tmp),"[%lu:%lu]",
+	        (unsigned long)slice->first,
+	        (unsigned long)((slice->first+slice->length)-1));
+    } else {
+	snprintf(tmp,sizeof(tmp),"[%lu:%lu:%lu]",
+		    (unsigned long)slice->first,
+		    (unsigned long)slice->stride,
+		    (unsigned long)((slice->first+slice->length)-1));
+    }
+    return tmp;
+}
+
 char*
 dumpslices(NCslice* slice, unsigned int rank)
 {
@@ -709,18 +713,7 @@ dumpslices(NCslice* slice, unsigned int rank)
 
     buf = ncbytesnew();
     for(i=0;i<rank;i++,slice++) {
-        char tmp[1024];
-        if(slice->stride == 1) {
-            snprintf(tmp,sizeof(tmp),"[%lu:%lu]",
-	        (unsigned long)slice->first,
-	        (unsigned long)((slice->first+slice->length)-1));
-        } else {
-	    snprintf(tmp,sizeof(tmp),"[%lu:%lu:%lu]",
-		    (unsigned long)slice->first,
-		    (unsigned long)slice->stride,
-		    (unsigned long)((slice->first+slice->length)-1));
-	}
-        ncbytescat(buf,tmp);
+        ncbytescat(buf,dumpslice(slice));
     }
     result = ncbytesdup(buf);
     ncbytesfree(buf);
