@@ -1,18 +1,18 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/libncdap3/getvara3.c,v 1.40 2009/12/03 18:53:16 dmh Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/libncdap3/getvara3.c,v 1.44 2010/05/27 21:34:08 dmh Exp $
  *********************************************************************/
 #include "ncdap3.h"
 #include "dapodom.h"
 #include "dapdump.h"
-#include "dapdispatch3.h"
+#include "ncd3dispatch.h"
 
 #define NEWVARM
 
 /* Forward:*/
-static NCerror moveto(NCDRNO*, Getvara*, CDFnode* dataroot, void* memory);
-static NCerror movetor(NCDRNO*, OCdata currentcontent,
+static NCerror moveto(NCDAPCOMMON*, Getvara*, CDFnode* dataroot, void* memory);
+static NCerror movetor(NCDAPCOMMON*, OCdata currentcontent,
 		   NClist* path, int depth,
 		   Getvara*, int dimindex,
 		   struct NCMEMORY*, NClist* segments);
@@ -33,31 +33,28 @@ nc3d_getvarx(int ncid, int varid,
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
     int i;
-    NC* ncp;
-    NCDRNO* drno;
+    NCDAP3* drno;
     NC_var* var;
     CDFnode* cdfvar; /* cdf node mapping to var*/
     NClist* varnodes;
     nc_type dsttype;
-    Getvara* varainfo = NULL;
+    Getvara* varinfo = NULL;
     CDFnode* xtarget = NULL; /* target in DATADDS */
     CDFnode* target = NULL; /* target in constrained DDS */
     NCprojection* varaprojection = NULL;
     NCcachenode* cachenode = NULL;
-    size_t localcount[NC_MAX_DIMS];
+    size_t localcount[NC_MAX_VAR_DIMS];
     NClist* vars = nclistnew();
-    NCconstraint constraint = {NULL,NULL};
+    NCconstraint* constraint = NULL;
 
-    ncstat = NC_check_id(ncid, &ncp); 
+    ncstat = NC_check_id(ncid, (NC**)&drno); 
     if(ncstat != NC_NOERR) goto fail;
-
-    drno = ncp->drno;
     
-    var = NC_lookupvar(ncp,varid);
+    var = NC_lookupvar((NC*)drno,varid);
     if(var == NULL) {ncstat = NC_ENOTVAR; goto fail;}
 
     /* Locate var node via varid */
-    varnodes = drno->cdf.varnodes;
+    varnodes = drno->dap.cdf.varnodes;
     for(i=0;i<nclistlength(varnodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(varnodes,i);
 	if(node->array.basevar == NULL
@@ -120,56 +117,57 @@ fprintf(stderr,"\n");
 	}
     }
 
-    ncstat = makegetvar34(drno,cdfvar,data,dsttype,&varainfo);
+    ncstat = makegetvar34(&drno->dap,cdfvar,data,dsttype,&varinfo);
     if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto fail;}
 #ifdef IGNORE
-    freegetvara(drno->cdf.vara);
-    drno->cdf.vara = varainfo;
+    freegetvara(drno->dap.cdf.vara);
+    drno->dap.cdf.vara = varinfo;
 #endif
 
-    ncstat = buildvaraprojection3(drno,varainfo,startp,countp,stridep,&varaprojection);
+    ncstat = buildvaraprojection3(varinfo,startp,countp,stridep,&varaprojection);
     if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto fail;}
 
-    if(FLAGSET(drno,NCF_UNCONSTRAINABLE)) {
+    if(FLAGSET(drno->dap.controls,NCF_UNCONSTRAINABLE)) {
 #ifdef DEBUG
 fprintf(stderr,"Unconstrained: reusing prefetch\n");
 #endif
-	cachenode = drno->cdf.cache.prefetch;	
+	cachenode = drno->dap.cdf.cache->prefetch;	
 	ASSERT((cachenode != NULL));
-    } else if(iscached(drno,varaprojection->leaf,&cachenode)) {
+    } else if(iscached(&drno->dap,varaprojection->var->leaf,&cachenode)) {
+	/* If it is cached, then it is a whole variable */
 #ifdef DEBUG
-fprintf(stderr,"Reusing cached fetch constraint: %s\n",
-	dumpconstraint(&cachenode->constraint));
+fprintf(stderr,"Reusing cache\n");
 #endif
-    } else { /* load with constraints */
-	nclistpush(vars,(ncelem)varainfo->target);
-        constraint.projections = cloneprojections(drno->dap.constraint.projections);
-        if(!FLAGSET(drno,NCF_CACHE)) {
+    } else { /*not cached: load using constraints */
+	nclistpush(vars,(ncelem)varinfo->target);
+	constraint = createncconstraint();
+        constraint->projections = clonencprojections(drno->dap.oc.dapconstraint->projections);
+        if(!FLAGSET(drno->dap.controls,NCF_CACHE)) {
 	    /* If we are not caching, then merge the getvara projections */
 	    NClist* tmp = nclistnew();
-	    NCprojection* clone = cloneprojection1(varaprojection);
+	    NCprojection* clone = clonencprojection(varaprojection);
 	    /* We need to modify the clone to remove
                pseudo dimensions (i.e. string and sequence dimensions)
                because the server will not recognize them
 	    */
-
 	    removepseudodims3(clone);
 	    nclistpush(tmp,(ncelem)clone);
-            ncstat = mergeprojections3(drno,constraint.projections,tmp);
-	    freencprojection1(clone);
+            ncstat = mergeprojections3(constraint->projections,tmp);
+	    freencprojection(clone);
 	    nclistfree(tmp);
             if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto fail;}
 #ifdef DEBUG
 fprintf(stderr,"vara merge: %s\n",
-	dumpprojections(constraint.projections));
+	dumpprojections(constraint->projections));
 #endif
         }
 
-        restrictprojection3(drno,vars,constraint.projections);
-        constraint.selections = cloneselections(drno->dap.constraint.selections);
+        restrictprojection34(vars,constraint->projections);
+        constraint->selections = clonencselections(drno->dap.oc.dapconstraint->selections);
 
-	/* buildcachenode3 will also fetch the corresponding datadds */
-        ncstat = buildcachenode3(drno,&constraint,vars,&cachenode,0);
+	/* buildcachenode3 will create a new cachenode and
+           will also fetch the corresponding datadds */
+        ncstat = buildcachenode34(&drno->dap,constraint,vars,&cachenode,0);
         if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto fail;}
 
 #ifdef DEBUG
@@ -178,47 +176,45 @@ fprintf(stderr,"cache.datadds=%s\n",dumptree(cachenode->datadds));
     }
 
     /* attach DATADDS to DDS */
-    unattach34(drno->cdf.ddsroot);
-    ncstat = attachsubset34(cachenode->datadds,drno->cdf.ddsroot);
+    unattach34(drno->dap.cdf.ddsroot);
+    ncstat = attachsubset34(cachenode->datadds,drno->dap.cdf.ddsroot);
     if(ncstat) goto fail;	
 
-    /* Fix up varainfo to use the cache */
-    varainfo->cache = cachenode;
+    /* Fix up varinfo to use the cache */
+    varinfo->cache = cachenode;
     /* However use this current vara projection */
-    varainfo->varaprojection = varaprojection;
+    varinfo->varaprojection = varaprojection;
     varaprojection = NULL;
 
     /* Now, walk to the relevant instance */
 
     /* Get the var correlate from the datadds */
-    target = varainfo->target;
+    target = varinfo->target;
     xtarget = target->attachment;
     if(xtarget == NULL) 
 	{THROWCHK(ncstat=NC_ENODATA); goto fail;}
 
     /* Switch to datadds tree space*/
-    varainfo->target = xtarget;
-    ncstat = moveto(drno,varainfo,cachenode->datadds,data);
+    varinfo->target = xtarget;
+    ncstat = moveto(&drno->dap,varinfo,cachenode->datadds,data);
     if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto fail;}
     goto ok;
 fail:
     if(ocstat != OC_NOERR) ncstat = ocerrtoncerr(ocstat);
-    freencprojection1(varaprojection);
+    freencprojection(varaprojection);
 ok:
-    clearncconstraint(&constraint);
+    freencconstraint(constraint);
     nclistfree(vars);
-    if(!FLAGSET(drno,NCF_UNCONSTRAINABLE) && !FLAGSET(drno,NCF_CACHE))
-	freenccachenode(drno,cachenode);
-    freegetvara(varainfo);
+    freegetvara(varinfo);
     return THROW(ncstat);
 }
 
 static NCerror
-moveto(NCDRNO* drno, Getvara* xgetvar, CDFnode* xrootnode, void* memory)
+moveto(NCDAPCOMMON* nccomm, Getvara* xgetvar, CDFnode* xrootnode, void* memory)
 {
     OCerror ocstat = OC_NOERR;
     NCerror ncstat = NC_NOERR;
-    OCconnection conn = drno->dap.conn;
+    OCconnection conn = nccomm->oc.conn;
     OCdata xrootcontent;
     OCobject ocroot;
     NClist* path = nclistnew();
@@ -234,9 +230,9 @@ moveto(NCDRNO* drno, Getvara* xgetvar, CDFnode* xrootnode, void* memory)
 
     /* Remember: xgetvar->target is in DATADDS tree */
     collectnodepath3(xgetvar->target,path,WITHDATASET);
-    ncstat = movetor(drno,xrootcontent,
+    ncstat = movetor(nccomm,xrootcontent,
                      path,0,xgetvar,0,&memstate,
-                     xgetvar->varaprojection->segments);
+                     xgetvar->varaprojection->var->segments);
 
 fail:
     nclistfree(path);
@@ -246,7 +242,7 @@ fail:
 }
 
 static NCerror
-movetor(NCDRNO* drno,
+movetor(NCDAPCOMMON* nccomm,
 	OCdata currentcontent,
 	NClist* path, int depth, /* depth is position in segment list*/
 	Getvara* xgetvar, int dimindex, /* dimindex is position in xgetvar->slices*/
@@ -257,7 +253,7 @@ movetor(NCDRNO* drno,
     OCerror ocstat = OC_NOERR;
     NCerror ncstat = NC_NOERR;
     size_t fieldindex,gridindex,rank;
-    OCconnection conn = drno->dap.conn;
+    OCconnection conn = nccomm->oc.conn;
     CDFnode* xnode = (CDFnode*)nclistget(path,depth);
     OCdata reccontent = OCNULL;
     OCdata dimcontent = OCNULL;
@@ -270,8 +266,8 @@ movetor(NCDRNO* drno,
     NCslice stringslice;
     NCsegment* segment;
     int newdepth;
-    int caching = FLAGSET(drno,NCF_CACHE);
-    int unconstrainable = FLAGSET(drno,NCF_UNCONSTRAINABLE);
+    int caching = FLAGSET(nccomm->controls,NCF_CACHE);
+    int unconstrainable = FLAGSET(nccomm->controls,NCF_UNCONSTRAINABLE);
     int wholepoint;
     size_t externtypesize;
     size_t interntypesize;
@@ -299,7 +295,7 @@ movetor(NCDRNO* drno,
 
     case CASE(NC_Dataset,OCFIELDMODE):
     case CASE(NC_Grid,OCFIELDMODE):
-    case CASE(NC_Structure,OCFIELDMODE):
+    case CASE(NC_Structure,OCFIELDMODE): /* fall thru */
     case CASE(NC_Sequence,OCFIELDMODE):
 	/* Since these are never dimensioned, we can go directly to
 	   the appropriate field; locate the field index for the next
@@ -327,7 +323,7 @@ movetor(NCDRNO* drno,
         fieldcontent = oc_data_new(conn);
         ocstat = oc_data_ith(conn,currentcontent,fieldindex,fieldcontent);
 	if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto fail;}
-	ncstat = movetor(drno,fieldcontent,
+	ncstat = movetor(nccomm,fieldcontent,
                          path,newdepth,xgetvar,dimindex,memory,
 			 segments);
 	break;
@@ -350,7 +346,7 @@ movetor(NCDRNO* drno,
             if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto fail;}
             ocstat = oc_data_mode(conn,dimcontent,&mode);
             ASSERT((mode == OCFIELDMODE));
-            ncstat = movetor(drno,dimcontent,
+            ncstat = movetor(nccomm,dimcontent,
                                  path,depth,
                                  xgetvar,dimindex+rank,
                                  memory,segments);
@@ -387,7 +383,7 @@ movetor(NCDRNO* drno,
                 THROWCHK(ocstat);
                 goto fail;
             }
-            ncstat = movetor(drno,reccontent,
+            ncstat = movetor(nccomm,reccontent,
                                  path,depth,
                                  xgetvar,dimindex+1,
                                  memory,segments);
@@ -421,12 +417,8 @@ movetor(NCDRNO* drno,
 
         rank = segment->slicerank;
 
-        /* whole variable is assumed to always be available
-	   (because currently we do not attempt to fold get_vars parameters
-            into the fetch projections)
-	*/
-		
-	if(caching || unconstrainable) {	
+	ASSERT(xgetvar->cache != NULL);
+	if(xgetvar->cache->wholevariable) {
             if(hasstringdim) {
 	        /* Get the string dimension */
 	        stringslice = segment->slices[rank-1];
@@ -436,7 +428,7 @@ movetor(NCDRNO* drno,
 	    } else {
                 odom = newdapodometer(segment->slices,0,rank);
 	    }
-	} else {/* ! caching */
+	} else { /*!xgetvar->cache->wholevariable*/
             if(hasstringdim) {
 	        /* Get the string dimension */
 	        stringslice = segment->slices[rank-1];
@@ -640,12 +632,12 @@ removepseudodims3(NCprojection* clone)
     NCsegment* seg;
 
     ASSERT((clone != NULL));
-    nsegs = nclistlength(clone->segments);
+    nsegs = nclistlength(clone->var->segments);
 
     /* 1. scan for sequences and remove
 	  any index projections. */
     for(i=0;i<nsegs;i++) {
-	seg = (NCsegment*)nclistget(clone->segments,i);
+	seg = (NCsegment*)nclistget(clone->var->segments,i);
 	if(seg->node->nctype != NC_Sequence) continue; /* not a sequence */
 	seg->slicerank = 0;
     }
@@ -653,7 +645,7 @@ removepseudodims3(NCprojection* clone)
     /* 2. Check the terminal segment to see if it is a String primitive,
           and if so, then remove the string dimension */
     ASSERT((nsegs > 0));
-    seg = (NCsegment*)nclistget(clone->segments,nsegs-1);
+    seg = (NCsegment*)nclistget(clone->var->segments,nsegs-1);
     /* See if the node has a string dimension */
     if(seg->node->nctype == NC_Primitive
        && seg->node->array.stringdim != NULL) {
@@ -673,14 +665,13 @@ nc3d_getvarmx(int ncid, int varid,
 {
     NCerror ncstat = NC_NOERR;
     int i;
-    NC* ncp;
-    NCDRNO* drno;
+    NCDAP3* drno;
     NC_var* var;
     CDFnode* cdfvar; /* cdf node mapping to var*/
     NClist* varnodes;
     nc_type dsttype;
     size_t externsize;
-    size_t dimsizes[NC_MAX_DIMS];
+    size_t dimsizes[NC_MAX_VAR_DIMS];
     Dapodometer* odom = NULL;
     unsigned int ncrank;
     NClist* ncdims = NULL;
@@ -689,16 +680,14 @@ nc3d_getvarmx(int ncid, int varid,
     char* localcopy; /* of whole variable */
 #endif
 
-    ncstat = NC_check_id(ncid, &ncp); 
+    ncstat = NC_check_id(ncid, (NC**)&drno); 
     if(ncstat != NC_NOERR) goto done;
 
-    drno = ncp->drno;
-    
-    var = NC_lookupvar(ncp,varid);
+    var = NC_lookupvar((NC*)drno,varid);
     if(var == NULL) {ncstat = NC_ENOTVAR; goto done;}
 
     /* Locate var node via varid */
-    varnodes = drno->cdf.varnodes;
+    varnodes = drno->dap.cdf.varnodes;
     for(i=0;i<nclistlength(varnodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(varnodes,i);
 	if(node->array.basevar == NULL
@@ -716,7 +705,10 @@ nc3d_getvarmx(int ncid, int varid,
        /* The variable is a scalar; consequently, there is only one
           thing to get and only one place to put it.  (Why was I
           called?) */
-       return nc3d_getvar(ncid, varid, data, dsttype0);
+	/* recurse with additional parameters */
+        return THROW(nc3d_getvarx(ncid,varid,
+		 dapzerostart3,NULL,dapsinglestride3,
+		 data,dsttype0));
     }
          
     dsttype = (dsttype0);
@@ -766,27 +758,27 @@ nc3d_getvarmx(int ncid, int varid,
     switch (dsttype) {
 
     case NC_CHAR:
-	ncstat = nc3d_get_vars_text(ncid,varid,start, edges, stride,
+	ncstat = nc_get_vars_text(ncid,varid,start, edges, stride,
 				  (char*)localcopy);
 	break;
     case NC_BYTE:
-	ncstat = nc3d_get_vars_schar(ncid,varid,start, edges, stride,
+	ncstat = nc_get_vars_schar(ncid,varid,start, edges, stride,
 				   (signed char*)localcopy);
 	break;
     case NC_SHORT:
-	ncstat = nc3d_get_vars_short(ncid,varid, start, edges, stride,
+	ncstat = nc_get_vars_short(ncid,varid, start, edges, stride,
 			  	   (short*)localcopy);
 	break;
     case NC_INT:
-	ncstat = nc3d_get_vars_int(ncid,varid,start, edges, stride,
+	ncstat = nc_get_vars_int(ncid,varid,start, edges, stride,
 				 (int*)localcopy);
 	break;
     case NC_FLOAT:
-	ncstat = nc3d_get_vars_float(ncid,varid,start, edges, stride,
+	ncstat = nc_get_vars_float(ncid,varid,start, edges, stride,
 				   (float*)localcopy);
 	break;
     case NC_DOUBLE:
-	ncstat = nc3d_get_vars_double(ncid,varid,	start, edges, stride,
+	ncstat = nc_get_vars_double(ncid,varid,	start, edges, stride,
 		 		    (double*)localcopy);
 	break;
     default: break;
@@ -819,7 +811,7 @@ fprintf(stderr,"new: %lu -> %lu  %f\n",
 	void* dataoffset = (void*)(((char*)data) + (externsize*voffset));
 
 	/* get the indexset'th value using variable's internal type */
-	ncstat = nc3d_get_var1(ncid,varid,indexset,(void*)&internalmem);
+	ncstat = nc_get_var1(ncid,varid,indexset,(void*)&internalmem);
         if(ncstat != NC_NOERR) goto done;
 	/* Convert to external type */
 	ncstat = dapconvert3(cdfvar->etype,dsttype,externalmem,internalmem);
