@@ -3,6 +3,7 @@
 #include <netcdf.h>
 
 
+
 void
 check_err(const int stat, const int line, const char *file) {
     if (stat != NC_NOERR) {
@@ -654,7 +655,7 @@ main() {/* create ctest0.nc */
 
     /* assign global attributes */
     { /* Gc */
-    stat = nc_put_att_text(ncid, NC_GLOBAL, "Gc", 1, "\000");
+    stat = nc_put_att_text(ncid, NC_GLOBAL, "Gc", 0, "");
     check_err(stat,__LINE__,__FILE__);
     }
     { /* Gb */
@@ -726,7 +727,7 @@ main() {/* create ctest0.nc */
     check_err(stat,__LINE__,__FILE__);
     }
     { /* c */
-    stat = nc_put_att_text(ncid, b_id, "c", 1, "\000");
+    stat = nc_put_att_text(ncid, b_id, "c", 0, "");
     check_err(stat,__LINE__,__FILE__);
     }
     { /* b */
@@ -800,11 +801,10 @@ main() {/* create ctest0.nc */
     stat = nc_put_var1(ncid, d_id, &zero, d_data);    check_err(stat,__LINE__,__FILE__);
     }
     {
-    char cr_data[2] = {"ab"} ;
+    char cr_data[2] = "ab" ;
     size_t cr_startset[1] = {0} ;
     size_t cr_countset[1] = {2} ;
     stat = nc_put_vara(ncid, cr_id, cr_startset, cr_countset, cr_data);
-    stat = nc_put_vara(ncid, cr_id, cr_startset, cr_countset, cr_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -813,7 +813,6 @@ main() {/* create ctest0.nc */
     size_t br_startset[1] = {0} ;
     size_t br_countset[1] = {2} ;
     stat = nc_put_vara(ncid, br_id, br_startset, br_countset, br_data);
-    stat = nc_put_vara(ncid, br_id, br_startset, br_countset, br_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -822,7 +821,6 @@ main() {/* create ctest0.nc */
     size_t sr_startset[1] = {0} ;
     size_t sr_countset[1] = {2} ;
     stat = nc_put_vara(ncid, sr_id, sr_startset, sr_countset, sr_data);
-    stat = nc_put_vara(ncid, sr_id, sr_startset, sr_countset, sr_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -831,7 +829,6 @@ main() {/* create ctest0.nc */
     size_t ir_startset[1] = {0} ;
     size_t ir_countset[1] = {2} ;
     stat = nc_put_vara(ncid, ir_id, ir_startset, ir_countset, ir_data);
-    stat = nc_put_vara(ncid, ir_id, ir_startset, ir_countset, ir_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -840,7 +837,6 @@ main() {/* create ctest0.nc */
     size_t fr_startset[1] = {0} ;
     size_t fr_countset[1] = {2} ;
     stat = nc_put_vara(ncid, fr_id, fr_startset, fr_countset, fr_data);
-    stat = nc_put_vara(ncid, fr_id, fr_startset, fr_countset, fr_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -849,16 +845,14 @@ main() {/* create ctest0.nc */
     size_t dr_startset[1] = {0} ;
     size_t dr_countset[1] = {2} ;
     stat = nc_put_vara(ncid, dr_id, dr_startset, dr_countset, dr_data);
-    stat = nc_put_vara(ncid, dr_id, dr_startset, dr_countset, dr_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char c1_data[1] = {"\000"} ;
+    char c1_data[1] = "\000" ;
     size_t c1_startset[1] = {0} ;
     size_t c1_countset[1] = {1} ;
     stat = nc_put_vara(ncid, c1_id, c1_startset, c1_countset, c1_data);
-    stat = nc_put_vara(ncid, c1_id, c1_startset, c1_countset, c1_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -867,7 +861,6 @@ main() {/* create ctest0.nc */
     size_t b1_startset[1] = {0} ;
     size_t b1_countset[1] = {1} ;
     stat = nc_put_vara(ncid, b1_id, b1_startset, b1_countset, b1_data);
-    stat = nc_put_vara(ncid, b1_id, b1_startset, b1_countset, b1_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -876,7 +869,6 @@ main() {/* create ctest0.nc */
     size_t s1_startset[1] = {0} ;
     size_t s1_countset[1] = {1} ;
     stat = nc_put_vara(ncid, s1_id, s1_startset, s1_countset, s1_data);
-    stat = nc_put_vara(ncid, s1_id, s1_startset, s1_countset, s1_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -885,7 +877,6 @@ main() {/* create ctest0.nc */
     size_t i1_startset[1] = {0} ;
     size_t i1_countset[1] = {1} ;
     stat = nc_put_vara(ncid, i1_id, i1_startset, i1_countset, i1_data);
-    stat = nc_put_vara(ncid, i1_id, i1_startset, i1_countset, i1_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -894,7 +885,6 @@ main() {/* create ctest0.nc */
     size_t f1_startset[1] = {0} ;
     size_t f1_countset[1] = {1} ;
     stat = nc_put_vara(ncid, f1_id, f1_startset, f1_countset, f1_data);
-    stat = nc_put_vara(ncid, f1_id, f1_startset, f1_countset, f1_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -903,16 +893,14 @@ main() {/* create ctest0.nc */
     size_t d1_startset[1] = {0} ;
     size_t d1_countset[1] = {1} ;
     stat = nc_put_vara(ncid, d1_id, d1_startset, d1_countset, d1_data);
-    stat = nc_put_vara(ncid, d1_id, d1_startset, d1_countset, d1_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char c2_data[2] = {"ab"} ;
+    char c2_data[2] = "ab" ;
     size_t c2_startset[1] = {0} ;
     size_t c2_countset[1] = {2} ;
     stat = nc_put_vara(ncid, c2_id, c2_startset, c2_countset, c2_data);
-    stat = nc_put_vara(ncid, c2_id, c2_startset, c2_countset, c2_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -921,7 +909,6 @@ main() {/* create ctest0.nc */
     size_t b2_startset[1] = {0} ;
     size_t b2_countset[1] = {2} ;
     stat = nc_put_vara(ncid, b2_id, b2_startset, b2_countset, b2_data);
-    stat = nc_put_vara(ncid, b2_id, b2_startset, b2_countset, b2_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -930,7 +917,6 @@ main() {/* create ctest0.nc */
     size_t s2_startset[1] = {0} ;
     size_t s2_countset[1] = {2} ;
     stat = nc_put_vara(ncid, s2_id, s2_startset, s2_countset, s2_data);
-    stat = nc_put_vara(ncid, s2_id, s2_startset, s2_countset, s2_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -939,7 +925,6 @@ main() {/* create ctest0.nc */
     size_t i2_startset[1] = {0} ;
     size_t i2_countset[1] = {2} ;
     stat = nc_put_vara(ncid, i2_id, i2_startset, i2_countset, i2_data);
-    stat = nc_put_vara(ncid, i2_id, i2_startset, i2_countset, i2_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -948,7 +933,6 @@ main() {/* create ctest0.nc */
     size_t f2_startset[1] = {0} ;
     size_t f2_countset[1] = {2} ;
     stat = nc_put_vara(ncid, f2_id, f2_startset, f2_countset, f2_data);
-    stat = nc_put_vara(ncid, f2_id, f2_startset, f2_countset, f2_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -957,16 +941,14 @@ main() {/* create ctest0.nc */
     size_t d2_startset[1] = {0} ;
     size_t d2_countset[1] = {2} ;
     stat = nc_put_vara(ncid, d2_id, d2_startset, d2_countset, d2_data);
-    stat = nc_put_vara(ncid, d2_id, d2_startset, d2_countset, d2_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char c3_data[3] = {"\001."} ;
+    char c3_data[3] = "\001." ;
     size_t c3_startset[1] = {0} ;
     size_t c3_countset[1] = {3} ;
     stat = nc_put_vara(ncid, c3_id, c3_startset, c3_countset, c3_data);
-    stat = nc_put_vara(ncid, c3_id, c3_startset, c3_countset, c3_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -975,7 +957,6 @@ main() {/* create ctest0.nc */
     size_t b3_startset[1] = {0} ;
     size_t b3_countset[1] = {3} ;
     stat = nc_put_vara(ncid, b3_id, b3_startset, b3_countset, b3_data);
-    stat = nc_put_vara(ncid, b3_id, b3_startset, b3_countset, b3_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -984,7 +965,6 @@ main() {/* create ctest0.nc */
     size_t s3_startset[1] = {0} ;
     size_t s3_countset[1] = {3} ;
     stat = nc_put_vara(ncid, s3_id, s3_startset, s3_countset, s3_data);
-    stat = nc_put_vara(ncid, s3_id, s3_startset, s3_countset, s3_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -993,7 +973,6 @@ main() {/* create ctest0.nc */
     size_t i3_startset[1] = {0} ;
     size_t i3_countset[1] = {3} ;
     stat = nc_put_vara(ncid, i3_id, i3_startset, i3_countset, i3_data);
-    stat = nc_put_vara(ncid, i3_id, i3_startset, i3_countset, i3_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1002,7 +981,6 @@ main() {/* create ctest0.nc */
     size_t f3_startset[1] = {0} ;
     size_t f3_countset[1] = {3} ;
     stat = nc_put_vara(ncid, f3_id, f3_startset, f3_countset, f3_data);
-    stat = nc_put_vara(ncid, f3_id, f3_startset, f3_countset, f3_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1011,16 +989,14 @@ main() {/* create ctest0.nc */
     size_t d3_startset[1] = {0} ;
     size_t d3_countset[1] = {3} ;
     stat = nc_put_vara(ncid, d3_id, d3_startset, d3_countset, d3_data);
-    stat = nc_put_vara(ncid, d3_id, d3_startset, d3_countset, d3_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char cr1_data[2] = {"xy"} ;
+    char cr1_data[2] = "xy" ;
     size_t cr1_startset[2] = {0, 0} ;
     size_t cr1_countset[2] = {2, 1} ;
     stat = nc_put_vara(ncid, cr1_id, cr1_startset, cr1_countset, cr1_data);
-    stat = nc_put_vara(ncid, cr1_id, cr1_startset, cr1_countset, cr1_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1029,7 +1005,6 @@ main() {/* create ctest0.nc */
     size_t br2_startset[2] = {0, 0} ;
     size_t br2_countset[2] = {2, 2} ;
     stat = nc_put_vara(ncid, br2_id, br2_startset, br2_countset, br2_data);
-    stat = nc_put_vara(ncid, br2_id, br2_startset, br2_countset, br2_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1038,7 +1013,6 @@ main() {/* create ctest0.nc */
     size_t sr3_startset[2] = {0, 0} ;
     size_t sr3_countset[2] = {2, 3} ;
     stat = nc_put_vara(ncid, sr3_id, sr3_startset, sr3_countset, sr3_data);
-    stat = nc_put_vara(ncid, sr3_id, sr3_startset, sr3_countset, sr3_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1047,7 +1021,6 @@ main() {/* create ctest0.nc */
     size_t f11_startset[2] = {0, 0} ;
     size_t f11_countset[2] = {1, 1} ;
     stat = nc_put_vara(ncid, f11_id, f11_startset, f11_countset, f11_data);
-    stat = nc_put_vara(ncid, f11_id, f11_startset, f11_countset, f11_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1056,16 +1029,14 @@ main() {/* create ctest0.nc */
     size_t d12_startset[2] = {0, 0} ;
     size_t d12_countset[2] = {1, 2} ;
     stat = nc_put_vara(ncid, d12_id, d12_startset, d12_countset, d12_data);
-    stat = nc_put_vara(ncid, d12_id, d12_startset, d12_countset, d12_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char c13_data[3] = {"\tb\177"} ;
+    char c13_data[3] = "\tb\177" ;
     size_t c13_startset[2] = {0, 0} ;
     size_t c13_countset[2] = {1, 3} ;
     stat = nc_put_vara(ncid, c13_id, c13_startset, c13_countset, c13_data);
-    stat = nc_put_vara(ncid, c13_id, c13_startset, c13_countset, c13_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1074,7 +1045,6 @@ main() {/* create ctest0.nc */
     size_t s21_startset[2] = {0, 0} ;
     size_t s21_countset[2] = {2, 1} ;
     stat = nc_put_vara(ncid, s21_id, s21_startset, s21_countset, s21_data);
-    stat = nc_put_vara(ncid, s21_id, s21_startset, s21_countset, s21_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1083,7 +1053,6 @@ main() {/* create ctest0.nc */
     size_t i22_startset[2] = {0, 0} ;
     size_t i22_countset[2] = {2, 2} ;
     stat = nc_put_vara(ncid, i22_id, i22_startset, i22_countset, i22_data);
-    stat = nc_put_vara(ncid, i22_id, i22_startset, i22_countset, i22_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1092,16 +1061,14 @@ main() {/* create ctest0.nc */
     size_t f23_startset[2] = {0, 0} ;
     size_t f23_countset[2] = {2, 3} ;
     stat = nc_put_vara(ncid, f23_id, f23_startset, f23_countset, f23_data);
-    stat = nc_put_vara(ncid, f23_id, f23_startset, f23_countset, f23_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char c31_data[3] = {"+- "} ;
+    char c31_data[3] = "+- " ;
     size_t c31_startset[2] = {0, 0} ;
     size_t c31_countset[2] = {3, 1} ;
     stat = nc_put_vara(ncid, c31_id, c31_startset, c31_countset, c31_data);
-    stat = nc_put_vara(ncid, c31_id, c31_startset, c31_countset, c31_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1110,7 +1077,6 @@ main() {/* create ctest0.nc */
     size_t b32_startset[2] = {0, 0} ;
     size_t b32_countset[2] = {3, 2} ;
     stat = nc_put_vara(ncid, b32_id, b32_startset, b32_countset, b32_data);
-    stat = nc_put_vara(ncid, b32_id, b32_startset, b32_countset, b32_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1119,7 +1085,6 @@ main() {/* create ctest0.nc */
     size_t s33_startset[2] = {0, 0} ;
     size_t s33_countset[2] = {3, 3} ;
     stat = nc_put_vara(ncid, s33_id, s33_startset, s33_countset, s33_data);
-    stat = nc_put_vara(ncid, s33_id, s33_startset, s33_countset, s33_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1128,7 +1093,6 @@ main() {/* create ctest0.nc */
     size_t sr11_startset[3] = {0, 0, 0} ;
     size_t sr11_countset[3] = {2, 1, 1} ;
     stat = nc_put_vara(ncid, sr11_id, sr11_startset, sr11_countset, sr11_data);
-    stat = nc_put_vara(ncid, sr11_id, sr11_startset, sr11_countset, sr11_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1137,7 +1101,6 @@ main() {/* create ctest0.nc */
     size_t ir12_startset[3] = {0, 0, 0} ;
     size_t ir12_countset[3] = {2, 1, 2} ;
     stat = nc_put_vara(ncid, ir12_id, ir12_startset, ir12_countset, ir12_data);
-    stat = nc_put_vara(ncid, ir12_id, ir12_startset, ir12_countset, ir12_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1146,16 +1109,14 @@ main() {/* create ctest0.nc */
     size_t fr13_startset[3] = {0, 0, 0} ;
     size_t fr13_countset[3] = {2, 1, 3} ;
     stat = nc_put_vara(ncid, fr13_id, fr13_startset, fr13_countset, fr13_data);
-    stat = nc_put_vara(ncid, fr13_id, fr13_startset, fr13_countset, fr13_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char cr21_data[4] = {"@DHL"} ;
+    char cr21_data[4] = "@DHL" ;
     size_t cr21_startset[3] = {0, 0, 0} ;
     size_t cr21_countset[3] = {2, 2, 1} ;
     stat = nc_put_vara(ncid, cr21_id, cr21_startset, cr21_countset, cr21_data);
-    stat = nc_put_vara(ncid, cr21_id, cr21_startset, cr21_countset, cr21_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1164,7 +1125,6 @@ main() {/* create ctest0.nc */
     size_t br22_startset[3] = {0, 0, 0} ;
     size_t br22_countset[3] = {2, 2, 2} ;
     stat = nc_put_vara(ncid, br22_id, br22_startset, br22_countset, br22_data);
-    stat = nc_put_vara(ncid, br22_id, br22_startset, br22_countset, br22_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1173,7 +1133,6 @@ main() {/* create ctest0.nc */
     size_t sr23_startset[3] = {0, 0, 0} ;
     size_t sr23_countset[3] = {2, 2, 3} ;
     stat = nc_put_vara(ncid, sr23_id, sr23_startset, sr23_countset, sr23_data);
-    stat = nc_put_vara(ncid, sr23_id, sr23_startset, sr23_countset, sr23_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1182,7 +1141,6 @@ main() {/* create ctest0.nc */
     size_t fr31_startset[3] = {0, 0, 0} ;
     size_t fr31_countset[3] = {2, 3, 1} ;
     stat = nc_put_vara(ncid, fr31_id, fr31_startset, fr31_countset, fr31_data);
-    stat = nc_put_vara(ncid, fr31_id, fr31_startset, fr31_countset, fr31_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1191,25 +1149,22 @@ main() {/* create ctest0.nc */
     size_t dr32_startset[3] = {0, 0, 0} ;
     size_t dr32_countset[3] = {2, 3, 2} ;
     stat = nc_put_vara(ncid, dr32_id, dr32_startset, dr32_countset, dr32_data);
-    stat = nc_put_vara(ncid, dr32_id, dr32_startset, dr32_countset, dr32_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char cr33_data[18] = {"1\000\000two3\000\0004\000\0005\000\000six"} ;
+    char cr33_data[18] = "1\000\000two3\000\0004\000\0005\000\000six" ;
     size_t cr33_startset[3] = {0, 0, 0} ;
     size_t cr33_countset[3] = {2, 3, 3} ;
     stat = nc_put_vara(ncid, cr33_id, cr33_startset, cr33_countset, cr33_data);
-    stat = nc_put_vara(ncid, cr33_id, cr33_startset, cr33_countset, cr33_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char c111_data[1] = {"@"} ;
+    char c111_data[1] = "@" ;
     size_t c111_startset[3] = {0, 0, 0} ;
     size_t c111_countset[3] = {1, 1, 1} ;
     stat = nc_put_vara(ncid, c111_id, c111_startset, c111_countset, c111_data);
-    stat = nc_put_vara(ncid, c111_id, c111_startset, c111_countset, c111_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1218,7 +1173,6 @@ main() {/* create ctest0.nc */
     size_t b112_startset[3] = {0, 0, 0} ;
     size_t b112_countset[3] = {1, 1, 2} ;
     stat = nc_put_vara(ncid, b112_id, b112_startset, b112_countset, b112_data);
-    stat = nc_put_vara(ncid, b112_id, b112_startset, b112_countset, b112_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1227,7 +1181,6 @@ main() {/* create ctest0.nc */
     size_t s113_startset[3] = {0, 0, 0} ;
     size_t s113_countset[3] = {1, 1, 3} ;
     stat = nc_put_vara(ncid, s113_id, s113_startset, s113_countset, s113_data);
-    stat = nc_put_vara(ncid, s113_id, s113_startset, s113_countset, s113_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1236,7 +1189,6 @@ main() {/* create ctest0.nc */
     size_t f121_startset[3] = {0, 0, 0} ;
     size_t f121_countset[3] = {1, 2, 1} ;
     stat = nc_put_vara(ncid, f121_id, f121_startset, f121_countset, f121_data);
-    stat = nc_put_vara(ncid, f121_id, f121_startset, f121_countset, f121_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1245,16 +1197,14 @@ main() {/* create ctest0.nc */
     size_t d122_startset[3] = {0, 0, 0} ;
     size_t d122_countset[3] = {1, 2, 2} ;
     stat = nc_put_vara(ncid, d122_id, d122_startset, d122_countset, d122_data);
-    stat = nc_put_vara(ncid, d122_id, d122_startset, d122_countset, d122_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char c123_data[6] = {"one2\000\000"} ;
+    char c123_data[6] = "one2\000\000" ;
     size_t c123_startset[3] = {0, 0, 0} ;
     size_t c123_countset[3] = {1, 2, 3} ;
     stat = nc_put_vara(ncid, c123_id, c123_startset, c123_countset, c123_data);
-    stat = nc_put_vara(ncid, c123_id, c123_startset, c123_countset, c123_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1263,7 +1213,6 @@ main() {/* create ctest0.nc */
     size_t s131_startset[3] = {0, 0, 0} ;
     size_t s131_countset[3] = {1, 3, 1} ;
     stat = nc_put_vara(ncid, s131_id, s131_startset, s131_countset, s131_data);
-    stat = nc_put_vara(ncid, s131_id, s131_startset, s131_countset, s131_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1272,7 +1221,6 @@ main() {/* create ctest0.nc */
     size_t i132_startset[3] = {0, 0, 0} ;
     size_t i132_countset[3] = {1, 3, 2} ;
     stat = nc_put_vara(ncid, i132_id, i132_startset, i132_countset, i132_data);
-    stat = nc_put_vara(ncid, i132_id, i132_startset, i132_countset, i132_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1281,7 +1229,6 @@ main() {/* create ctest0.nc */
     size_t f133_startset[3] = {0, 0, 0} ;
     size_t f133_countset[3] = {1, 3, 3} ;
     stat = nc_put_vara(ncid, f133_id, f133_startset, f133_countset, f133_data);
-    stat = nc_put_vara(ncid, f133_id, f133_startset, f133_countset, f133_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1290,7 +1237,6 @@ main() {/* create ctest0.nc */
     size_t f211_startset[3] = {0, 0, 0} ;
     size_t f211_countset[3] = {2, 1, 1} ;
     stat = nc_put_vara(ncid, f211_id, f211_startset, f211_countset, f211_data);
-    stat = nc_put_vara(ncid, f211_id, f211_startset, f211_countset, f211_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1299,7 +1245,6 @@ main() {/* create ctest0.nc */
     size_t d212_startset[3] = {0, 0, 0} ;
     size_t d212_countset[3] = {2, 1, 2} ;
     stat = nc_put_vara(ncid, d212_id, d212_startset, d212_countset, d212_data);
-    stat = nc_put_vara(ncid, d212_id, d212_startset, d212_countset, d212_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1308,7 +1253,6 @@ main() {/* create ctest0.nc */
     size_t s221_startset[3] = {0, 0, 0} ;
     size_t s221_countset[3] = {2, 2, 1} ;
     stat = nc_put_vara(ncid, s221_id, s221_startset, s221_countset, s221_data);
-    stat = nc_put_vara(ncid, s221_id, s221_startset, s221_countset, s221_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1317,7 +1261,6 @@ main() {/* create ctest0.nc */
     size_t i222_startset[3] = {0, 0, 0} ;
     size_t i222_countset[3] = {2, 2, 2} ;
     stat = nc_put_vara(ncid, i222_id, i222_startset, i222_countset, i222_data);
-    stat = nc_put_vara(ncid, i222_id, i222_startset, i222_countset, i222_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1326,16 +1269,14 @@ main() {/* create ctest0.nc */
     size_t f223_startset[3] = {0, 0, 0} ;
     size_t f223_countset[3] = {2, 2, 3} ;
     stat = nc_put_vara(ncid, f223_id, f223_startset, f223_countset, f223_data);
-    stat = nc_put_vara(ncid, f223_id, f223_startset, f223_countset, f223_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
     {
-    char c231_data[6] = {"@DHHLP"} ;
+    char c231_data[6] = "@DHHLP" ;
     size_t c231_startset[3] = {0, 0, 0} ;
     size_t c231_countset[3] = {2, 3, 1} ;
     stat = nc_put_vara(ncid, c231_id, c231_startset, c231_countset, c231_data);
-    stat = nc_put_vara(ncid, c231_id, c231_startset, c231_countset, c231_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1344,7 +1285,6 @@ main() {/* create ctest0.nc */
     size_t b232_startset[3] = {0, 0, 0} ;
     size_t b232_countset[3] = {2, 3, 2} ;
     stat = nc_put_vara(ncid, b232_id, b232_startset, b232_countset, b232_data);
-    stat = nc_put_vara(ncid, b232_id, b232_startset, b232_countset, b232_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1353,7 +1293,6 @@ main() {/* create ctest0.nc */
     size_t s233_startset[3] = {0, 0, 0} ;
     size_t s233_countset[3] = {2, 3, 3} ;
     stat = nc_put_vara(ncid, s233_id, s233_startset, s233_countset, s233_data);
-    stat = nc_put_vara(ncid, s233_id, s233_startset, s233_countset, s233_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1362,7 +1301,6 @@ main() {/* create ctest0.nc */
     size_t s311_startset[3] = {0, 0, 0} ;
     size_t s311_countset[3] = {3, 1, 1} ;
     stat = nc_put_vara(ncid, s311_id, s311_startset, s311_countset, s311_data);
-    stat = nc_put_vara(ncid, s311_id, s311_startset, s311_countset, s311_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1371,7 +1309,6 @@ main() {/* create ctest0.nc */
     size_t i312_startset[3] = {0, 0, 0} ;
     size_t i312_countset[3] = {3, 1, 2} ;
     stat = nc_put_vara(ncid, i312_id, i312_startset, i312_countset, i312_data);
-    stat = nc_put_vara(ncid, i312_id, i312_startset, i312_countset, i312_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
@@ -1380,7 +1317,6 @@ main() {/* create ctest0.nc */
     size_t f313_startset[3] = {0, 0, 0} ;
     size_t f313_countset[3] = {3, 1, 3} ;
     stat = nc_put_vara(ncid, f313_id, f313_startset, f313_countset, f313_data);
-    stat = nc_put_vara(ncid, f313_id, f313_startset, f313_countset, f313_data);
     check_err(stat,__LINE__,__FILE__);
     }
 
