@@ -1,11 +1,14 @@
 /*********************************************************************
  *   Copyright 2008, University Corporation for Atmospheric Research
  *   See netcdf/README file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/ncdump/ncdump.c,v 1.152 2010/02/04 16:49:12 russ Exp $
+ *   $Id: ncdump.c 400 2010-08-27 21:02:52Z russ $
  *********************************************************************/
 
 #include <config.h>
 #include <stdio.h>
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 #ifndef _WIN32
 #include <unistd.h>
 #endif
@@ -25,6 +28,7 @@
 #include "indent.h"
 #include "isnan.h"
 #include "cdl.h"
+#include "utils.h"
 
 #define int64_t long long
 #define uint64_t unsigned long long
@@ -386,8 +390,6 @@ pr_att_valgs(
 #endif /* USE_NETCDF4 */
     char *delim = ", ";	/* delimiter between output values */
 
-    if (len == 0)
-	return;
     if (type == NC_CHAR) {
 	char *cp = (char *) vals;
 	pr_att_string(kind, len, cp);
@@ -646,17 +648,17 @@ pr_att(
 	  case NC_VLEN:
 	      /* because size returned for vlen is base type size, but we
 	       * need space to read array of vlen structs into ... */
-	     data = emalloc(att.len * sizeof(nc_vlen_t));
+	      data = emalloc((att.len + 1) * sizeof(nc_vlen_t));
 	     break;
 	  case NC_OPAQUE:
-	      data = emalloc(att.len * type_size);
+	      data = emalloc((att.len + 1) * type_size);
 	     break;
 	  case NC_ENUM:
 	      /* a long long is ample for all base types */
-	     data = emalloc(att.len * sizeof(int64_t));
+	      data = emalloc((att.len + 1) * sizeof(int64_t));
 	     break;
 	  case NC_COMPOUND:
-	      data = emalloc(att.len * type_size);
+	      data = emalloc((att.len + 1) * type_size);
 	     break;
 	  default:
 	     error("unrecognized class of user defined type: %d", class);
@@ -795,7 +797,7 @@ pr_att_specials(
 	   int i;
 	    pr_att_name(ncid, varp->name, NC_ATT_STORAGE);
 	    printf(" = \"chunked\" ;\n");
-	    chunkp = (size_t *) emalloc(sizeof(size_t) * varp->ndims + 1 );
+	    chunkp = (size_t *) emalloc(sizeof(size_t) * (varp->ndims + 1) );
 	    NC_CHECK( nc_inq_var_chunking(ncid, varid, NULL, chunkp) );
 	    /* print chunking, even if it is default */
 	    pr_att_name(ncid, varp->name, NC_ATT_CHUNKING);
@@ -972,7 +974,7 @@ pr_shape(ncvar_t* varp, ncdim_t *dims)
     for (id = 0; id < varp->ndims; id++) {
 	shapelen += strlen(dims[varp->dims[id]].name) + 1;
     }
-    shape = (char *) emalloc(shapelen);
+    shape = (char *) emalloc(shapelen + 1);
     shape[0] = '\0';
     for (id = 0; id < varp->ndims; id++) {
 	/* TODO: XML-ish escapes for special chars in dim names */
@@ -1106,7 +1108,7 @@ print_ud_type(int ncid, nc_type typeid) {
 	    char field_type_name[NC_MAX_NAME + 1];
 	    size_t field_offset;
 	    nc_type field_type;
-	    int field_ndims, field_dim_sizes[NC_MAX_DIMS];
+	    int field_ndims;
 	    int d;
 	    
 	    indent_out();
@@ -1117,8 +1119,8 @@ print_ud_type(int ncid, nc_type typeid) {
 	    for (f = 0; f < type_nfields; f++)
 		{
 		    NC_CHECK( nc_inq_compound_field(ncid, typeid, f, field_name, 
-						    &field_offset, &field_type, &field_ndims,
-						    field_dim_sizes) );
+						    &field_offset, &field_type, 
+						    &field_ndims, NULL) );
 		    /* TODO: don't bother if field_type_name not needed here */
 		    get_type_name(ncid, field_type, field_type_name);
 		    indent_out();
@@ -1128,10 +1130,15 @@ print_ud_type(int ncid, nc_type typeid) {
 		    printf(" ");
 		    print_name(field_name);
 		    if (field_ndims > 0) {
+			int *field_dim_sizes = (int *) emalloc((field_ndims + 1) * sizeof(int));
+			NC_CHECK( nc_inq_compound_field(ncid, typeid, f, NULL, 
+							NULL, NULL, NULL, 
+							field_dim_sizes) );
 			printf("(");
 			for (d = 0; d < field_ndims-1; d++)
 			    printf("%d, ", field_dim_sizes[d]);
 			printf("%d)", field_dim_sizes[field_ndims-1]);
+			free(field_dim_sizes);
 		    }
 		    printf(" ;\n");
 		}
@@ -1158,7 +1165,7 @@ get_fill_info(int ncid, int varid, ncvar_t *vp) {
 	    
     /* get _FillValue attribute */
     nc_status = nc_inq_att(ncid,varid,_FillValue,&att.type,&att.len);
-    fillvalp = emalloc(vp->tinfo->size);
+    fillvalp = emalloc(vp->tinfo->size + 1);
     if(nc_status == NC_NOERR &&
        att.type == vp->type && att.len == 1) {
 	NC_CHECK(nc_get_att(ncid, varid, _FillValue, fillvalp));
@@ -1359,7 +1366,7 @@ do_ncdump_rec(int ncid, const char *path, fspec_t* specp)
    {
       int t;
 
-      typeids = emalloc(ntypes * sizeof(int));
+      typeids = emalloc((ntypes + 1) * sizeof(int));
       NC_CHECK( nc_inq_typeids(ncid, &ntypes, typeids) );
       indent_out();
       printf("types:\n");
@@ -1497,10 +1504,46 @@ do_ncdump_rec(int ncid, const char *path, fspec_t* specp)
       if (var.ndims > 0)
 	 printf ("(");
       for (id = 0; id < var.ndims; id++) {
-/* 	 printf ("%s%s", dims[var.dims[id]].name, id < var.ndims-1 ? ", " : ")"); */
 	 /* This dim may be in a parent group, so let's look up the
 	  * name. */
 	 NC_CHECK( nc_inq_dimname(ncid, var.dims[id], dim_name) );
+#ifdef USE_NETCDF4
+	 /* Subtlety: The following code block is needed because
+	  * nc_inq_dimname() currently returns only a simple dimension
+	  * name, without a prefix identifying the group it came from.
+	  * That's OK unless the dimid identifies a dimension in an
+	  * ancestor group that has the same simple name as a
+	  * dimension in the current group (or some intermediate
+	  * group), in which case the simple name is ambiguous.  This
+	  * code tests for that case and provides an absolute dimname
+	  * only in the case where a simple name would be
+	  * ambiguous. */
+	 {
+	     int dimid_test;	/* to see if dimname is ambiguous */
+	     int locid;		/* group id where dimension is defined */
+	     NC_CHECK( nc_inq_dimid(ncid, dim_name, &dimid_test) );
+	     locid = ncid;
+	     while(var.dims[id] != dimid_test) { /* not in locid, try ancestors */
+		 int parent_id;
+		 NC_CHECK( nc_inq_grp_parent(locid, &parent_id) );
+		 locid = parent_id;
+		 NC_CHECK( nc_inq_dimid(locid, dim_name, &dimid_test) );
+	     }
+	     /* dimid is in group locid, prefix dimname with group name if needed */
+	     if(locid != ncid) {
+		 size_t len;
+		 char *locname;	/* the group name */
+		 NC_CHECK( nc_inq_grpname_full(locid, &len, NULL) );
+		 locname = emalloc(len + 1);
+		 NC_CHECK( nc_inq_grpname_full(locid, &len, locname) );
+		 print_name (locname);
+		 if(strcmp("/", locname) != 0) { /* not the root group */
+		     printf("/");		 /* ensure a trailing slash */
+		 }
+		 free(locname);
+	     }
+	 }
+#endif	/* USE_NETCDF4 */
 	 print_name (dim_name);
 	 printf ("%s", id < var.ndims-1 ? ", " : ")");
       }
@@ -1566,8 +1609,6 @@ do_ncdump_rec(int ncid, const char *path, fspec_t* specp)
 	 no_data = 0;
 	 for (id = 0; id < var.ndims; id++) {
 	     size_t len;
-	     /* bug, wrong length value returned if dimension
-	      * inherited from ancestor group */
 	     NC_CHECK( nc_inq_dimlen(ncid, var.dims[id], &len) );
 	     if(len == 0) {
 		 no_data = 1;
@@ -1614,7 +1655,7 @@ do_ncdump_rec(int ncid, const char *path, fspec_t* specp)
       char group_name[NC_MAX_NAME + 1];
 
       /* See how many groups there are. */
-      NC_CHECK( nc_status = nc_inq_grps(ncid, &numgrps, NULL) );
+      NC_CHECK( nc_inq_grps(ncid, &numgrps, NULL) );
       
       /* Allocate memory to hold the list of group ids. */
       ncids = emalloc((numgrps + 1) * sizeof(int));
@@ -1911,7 +1952,7 @@ nc_inq_varname_count(int ncid, char *varname) {
 
 #ifdef USE_NETCDF4
     /* if this group has subgroups, call recursively on each of them */
-    NC_CHECK( nc_status = nc_inq_grps(ncid, &numgrps, NULL) );
+    NC_CHECK( nc_inq_grps(ncid, &numgrps, NULL) );
 	 
     /* Allocate memory to hold the list of group ids. */
     ncids = emalloc((numgrps + 1) * sizeof(int));
@@ -1964,9 +2005,6 @@ void adapt_url_for_cache(char **pathp) {
 int
 main(int argc, char *argv[])
 {
-    extern int optind;
-    extern int opterr;
-    extern char *optarg;
     static fspec_t fspec =	/* defaults, overridden on command line */
       {
 	  0,			/* construct netcdf name from file name */
