@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/ncgen/genbin.c,v 1.1 2010/04/04 20:13:20 dmh Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/ncgen/genbin.c,v 1.4 2010/05/27 21:34:17 dmh Exp $
  *********************************************************************/
 
 #include <ctype.h>	/* for isprint() */
@@ -101,7 +101,7 @@ gen_netcdf(const char *filename)
             Symbol* vsym = (Symbol*)listget(vardefs,ivar);
 	    if (vsym->typ.dimset.ndims > 0) {	/* a dimensioned variable */
 		/* construct a vector of dimension ids*/
-		int dimids[NC_MAX_DIMS];
+		int dimids[NC_MAX_VAR_DIMS];
 		for(idim=0;idim<vsym->typ.dimset.ndims;idim++)
 		    dimids[idim] = vsym->typ.dimset.dimsyms[idim]->ncid;
 		stat = nc_def_var(vsym->container->ncid,
@@ -127,7 +127,7 @@ gen_netcdf(const char *filename)
     bindata_vlenconstants(vlenconstants);
 
     /* define special variable properties */
-    if(!usingclassic && nvars > 0) {
+    if(nvars > 0) {
 	for(ivar = 0; ivar < nvars; ivar++) {
             Symbol* var = (Symbol*)listget(vardefs,ivar);
 	    genbin_definespecialattributes(var);
@@ -349,9 +349,7 @@ genbin_defineattr(Symbol* asym,Bytebuffer* databuf)
     typid = basetype->ncid;
 
     list = asym->data;
-    if(list == NULL) PANIC("empty attribute list");
     len = list->length;
-    if(len == 0) PANIC("empty attribute list");
 
     bindata_attrdata(asym,databuf);
 
@@ -366,8 +364,10 @@ genbin_defineattr(Symbol* asym,Bytebuffer* databuf)
             case NC_CHAR: {
                 char* data = (char*)bbContents(databuf);
 		size_t slen = bbLength(databuf);
+#ifdef IGNORE
 		/* Revise length if slen == 0 */
 		if(slen == 0) {bbAppend(databuf,'\0'); slen++;}
+#endif
                 stat = nc_put_att_text(grpid,varid,asym->name,slen,data);
                 check_err(stat,__LINE__,__FILE__);  
             } break;
@@ -401,7 +401,7 @@ genbin_defineattr(Symbol* asym,Bytebuffer* databuf)
 		} break;
             case NC_UBYTE: {
                 unsigned char* data = (unsigned char*)bbContents(databuf);
-                stat = nc_put_att_ubyte(grpid,varid,asym->name,typid,len,data);
+                stat = nc_put_att_uchar(grpid,varid,asym->name,typid,len,data);
                 check_err(stat,__LINE__,__FILE__);  
             } break;
             case NC_USHORT: {
@@ -448,36 +448,43 @@ genbin_definevardata(Symbol* vsym)
     int varid, grpid;
     int rank;
     Bytebuffer* memory;
-    Datasrc* src;
-    int chartype = (vsym->typ.basetype->typ.typecode == NC_CHAR);
     nciter_t iter;
     Odometer* odom = NULL;
     size_t nelems;
+    int chartype = (vsym->typ.basetype->typ.typecode == NC_CHAR);
     Datalist* fillsrc = vsym->var.special._Fillvalue;
+    int isscalar = (vsym->typ.dimset.ndims == 0);
 
     grpid = vsym->container->ncid,
     varid = vsym->ncid;
     rank = vsym->typ.dimset.ndims;
 
     memory = bbNew();
-
     /* give the buffer a running start to be large enough*/
     bbSetalloc(memory, nciterbuffersize);
 
     if(vsym->data == NULL) return;
-    src = datalist2src(vsym->data);
-    if(vsym->typ.dimset.ndims == 0) { /*scalar */
-	bindata_basetype(vsym->typ.basetype,src,memory,fillsrc); /*scalar*/
-        if(bbLength(memory) > 0)
-	    genbin_write(vsym,memory,odom,1);
-    } else { /* This is the heavy lifing */
-        /* Create an iterator to generate blocks of data */
-        nc_get_iter(vsym,nciterbuffersize,&iter);
-        /* Fill in the local odometer instance */
+
+    /* Generate character constants separately */    
+    if(!isscalar && chartype) {
+        gen_chararray(vsym,memory,fillsrc);
+	/* generate a corresponding odometer */
         odom = newodometer(&vsym->typ.dimset,NULL,NULL);
-        for(;;) {
-	    nelems=nc_next_iter(&iter,odom->start,odom->count);
-	    if(nelems == 0) break;
+        genbin_write(vsym,memory,odom,0);
+    } else { /* not character constant */
+        Datasrc* src = datalist2src(vsym->data);
+        if(isscalar) { /*scalar */
+            bindata_basetype(vsym->typ.basetype,src,memory,fillsrc); /*scalar*/
+            if(bbLength(memory) > 0)
+                genbin_write(vsym,memory,odom,1);
+        } else { /* This is the heavy lifing */
+            /* Create an iterator to generate blocks of data */
+            nc_get_iter(vsym,nciterbuffersize,&iter);
+            /* Fill in the local odometer instance */
+            odom = newodometer(&vsym->typ.dimset,NULL,NULL);
+            for(;;) {
+                nelems=nc_next_iter(&iter,odom->start,odom->count);
+                if(nelems == 0) break;
 if(debug > 0) {/*dump the iteration info*/
 int i;
 fprintf(stderr,"iter: %s->start[",vsym->name);
@@ -488,16 +495,14 @@ for(i=0;i<odom->rank;i++)
 fprintf(stderr,"%s%lu",(i==0?"":","),(unsigned long)odom->count[i]);
 fprintf(stderr,"]\n");
 }
-            if(chartype) {/* Handle character case separately */
-                gen_chararray(vsym,memory,src,odom,0);
-    	    } else
                 bindata_array(vsym,memory,src,odom,/*index=*/0,fillsrc);
-   	    /* Dump this chunk of (non-scalar) memory */
-	    genbin_write(vsym,memory,odom,0);
-	}
-        /* Write any residual data */
-        if(bbLength(memory) > 0)
-	    genbin_write(vsym,memory,odom,0);
+                /* Dump this chunk of (non-scalar) memory */
+                genbin_write(vsym,memory,odom,0);
+            }
+            /* Write any residual data */
+            if(bbLength(memory) > 0)
+                genbin_write(vsym,memory,odom,0);
+        }
     }
     odometerfree(odom);
     bbFree(memory);
@@ -507,6 +512,7 @@ static void
 genbin_write(Symbol* vsym, Bytebuffer* memory, Odometer* odom, int scalar)
 {
     int stat = NC_NOERR;
+
 if(!scalar && debug > 0) {
     int i;
     fprintf(stderr,"startset = [");
