@@ -3,7 +3,7 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/ncgen/genlib.h,v 1.18 2010/04/04 19:39:48 dmh Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/ncgen/genlib.h,v 1.20 2010/05/17 23:26:45 dmh Exp $
  *********************************************************************/
 #include <stdlib.h>
 #include <limits.h>
@@ -90,6 +90,10 @@ extern  void semerror(const int, const char *fmt, ...);
 #else
 extern  void semerror(lno,fmt,va_alist) const int lno; const char* fmt; va_dcl;
 #endif
+extern int nounlimited(Dimset* dimset, int from);
+extern int lastunlimited(Dimset* dimset);
+extern void padstring(Constant* con, size_t desiredlength, int fillchar);
+
 
 extern Datalist* explodestrings(Datalist*,char*);
 extern Datalist* implodestrings(Datalist*,char*);
@@ -142,6 +146,7 @@ extern void jflush(void);
 
 /* from: main.c */
 extern int usingclassic;
+extern int allowspecial;
 
 /* Global data */
 
@@ -184,4 +189,6 @@ extern void* emalloc (size_t);
 extern void* ecalloc (size_t);
 extern void* erealloc(void*,size_t);
 
+extern char* specialname(int flag);
+
 #endif /*!NC_GENLIB_H*/
